from typing import Dict
from workspace import deps, message, git
import os
from colorama import Fore, Back, Style

def generate():
  message.bright('* Generating CMake')
  print('')

  cmake = "cmake_minimum_required(VERSION 3.0)\n"
  cmake += "project(workspace)\n\n"

  cmake += """
add_compile_options(
  $<$<CXX_COMPILER_ID:GNU>:-fdiagnostics-color=always>
  $<$<CXX_COMPILER_ID:Clang>:-fcolor-diagnostics>
)

"""

  for directory in git.get_directories():
    cmakes = ['']
    
    config = deps.get_config(directory)
    if config and 'cmakes' in config:
      cmakes = config['cmakes']
    
    for cmake_name in cmakes:
      cmake_directory = os.path.realpath(directory)
      dname = os.path.basename(os.path.dirname(directory))
      name = os.path.basename(directory)
      project = '%s_%s' % (name, dname)

      if cmake_name:
        project += '_' + cmake_name
        cmake_directory += '/' + cmake_name

      cmake += "add_subdirectory(%s %s)\n" % (cmake_directory, project)
      print('- %s [%s]' % (project, Style.DIM + cmake_directory + Style.RESET_ALL))
  
  f = open('CMakeLists.txt', 'w')
  f.write(cmake)
  f.close()
  message.bright("* Wrote CMakeLists.txt")
  print('')



import os
from workspace import env, git, message, cmake

name = 'build'
help = """Runs the build"""
usage = ""
min_args = 0

def run(args):
  message.bright('* Building')

  if not os.path.exists('build'):
    os.makedirs('build')
  
  cmd = 'cd build; cmake -G Ninja ..; ninja %s' % ' '.join(args)
  os.system(cmd)
  


import os
from colorama import Fore, Back, Style

def error(message):
  print(Style.BRIGHT + Fore.RED + message + Style.RESET_ALL)

def bright(message):
  print('')
  print(Style.BRIGHT + message + Style.RESET_ALL)

def run_or_fail(cmd):
  if os.system(cmd) != 0:
    error('Error while running '+cmd)
    exit()



import os
import yaml

def get_config(directory):
  deps_filename = directory + '/deps.yml'
  if os.path.exists(deps_filename):
    f = open(deps_filename, 'r')
    deps = yaml.load(f, Loader=yaml.Loader)

    return deps
    
  return None


from colorama import Fore, Back, Style
from workspace import commands

name = "help"
help = """Provides help about commands"""
usage = ""
min_args = 0

def run(args):
  print('Available commands:')
  print('')

  for entry in commands.commands:
    print(Style.BRIGHT + Fore.BLUE + "wks" + Fore.RESET + " " + entry.name + ' ' + Style.RESET_ALL + entry.usage)
    lines = entry.help.strip().split("\n")
    lines = ["    " + line for line in lines]
    print("\n".join(lines))
    print('')